Gibberish.docs = {"Gibberish.Synth":{"text":"<h1 id=\"gibberishsynthsynth\">Gibberish.Synth </h1>\n\n<p>Oscillator + attack / decay envelope.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>Gibberish.init(); <br />\na = new Gibberish.Synth({ attack:44, decay:44100 }).connect(); <br />\na.note(880); <br />\na.waveform = \"Triangle\"; <br />\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{"note":"<h3 id=\"gibberishsynthnotemethod\">Gibberish.Synth.note : method</h3>\n\n<p>Generate an enveloped note at the provided frequency  </p>\n\n<p>param <strong>frequency</strong> Number. The frequency for the oscillator. <br />\nparam <strong>amp</strong> Number. Optional. The volume to use.  </p>"},"properties":{"frequency":"<h3 id=\"gibberishsynthfrequencyproperty\">Gibberish.Synth.frequency : property</h3>\n\n<p>Number. The frequency for the carrier oscillator. This is normally set using the note method but can also be modulated.</p>","pulsewidth":"<h3 id=\"gibberishsynthpulsewidthproperty\">Gibberish.Synth.pulsewidth : property</h3>\n\n<p>Number. The duty cycle for PWM synthesis</p>","attack":"<h3 id=\"gibberishsynthattackproperty\">Gibberish.Synth.attack : property</h3>\n\n<p>Number. The length of the attack portion of the envelope in samples. Note that the synth's envelope affects both amplitude and the index of the synth.</p>","decay":"<h3 id=\"gibberishsynthdecayproperty\">Gibberish.Synth.decay : property</h3>\n\n<p>Number. The length of the decay portion of the envelope in samples. Note that the synth's envelope affects both amplitude and the index of the synth.</p>","glide":"<h3 id=\"gibberishsynthglideproperty\">Gibberish.Synth.glide : property</h3>\n\n<p>Number. The synth has a one-pole filter attached to the carrier frequency. Set glide to a value between .999 and 1 to get pitch sweep between notes.</p>","amp":"<h3 id=\"gibberishsynthampproperty\">Gibberish.Synth.amp : property</h3>\n\n<p>Number. The relative amplitude level of the synth.</p>","channels":"<h3 id=\"gibberishsynthchannelsproperty\">Gibberish.Synth.channels : property</h3>\n\n<p>Number. Default 2. Mono or Stereo synthesis.</p>","pan":"<h3 id=\"gibberishsynthpanproperty\">Gibberish.Synth.pan : property</h3>\n\n<p>Number. Default 0. If the synth has two channels, this determines its position in the stereo spectrum.</p>","waveform":"<h3 id=\"gibberishsynthwaveformproperty\">Gibberish.Synth.waveform : property</h3>\n\n<p>String. The type of waveform to use. Options include 'Sine', 'Triangle', 'PWM', 'Saw' etc.</p>"},"type":" Synth"},"Gibberish.PolySynth":{"text":"<h1 id=\"gibberishpolysynthsynth\">Gibberish.PolySynth </h1>\n\n<p>A polyphonic version of <a href=\"javascript:displayDocs('Gibberish.Synth')). There are two additional properties for the polyphonic version of the synth. The polyphonic version consists of multiple Synths being fed into a single [Bus](javascript:displayDocs('Gibberish.Bus')\">Synth</a> object.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>Gibberish.init(); <br />\na = new Gibberish.PolySytn({ attack:88200, decay:88200, maxVoices:10 }).connect(); <br />\na.note(880); <br />\na.note(1320); \na.note(1760); <br />\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p>One important property to pass to the constructor is the maxVoices property, which defaults to 5. This controls how many voices are allocated to the synth and cannot be changed after initialization.  </p>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{"note":"<h3 id=\"gibberishpolysynthnotemethod\">Gibberish.PolySynth.note : method</h3>\n\n<p>Generate an enveloped note at the provided frequency using a simple voice allocation system where if all children are active, the one active the longest cancels its current note and begins playing a new one.    </p>\n\n<p>param <strong>frequency</strong> Number. The frequency for the oscillator. \nparam <strong>amp</strong> Number. Optional. The volume to use.  </p>"},"properties":{"children":"<h3 id=\"gibberishpolysynthchildrenproperty\">Gibberish.PolySynth.children : property</h3>\n\n<p>Array. Read-only. An array holding all of the child FMSynth objects.</p>","maxVoices":"<h3 id=\"gibberishpolysynthmaxvoicesproperty\">Gibberish.PolySynth.maxVoices : property</h3>\n\n<p>Number. The number of voices of polyphony the synth has. May only be set in initialization properties passed to constrcutor.</p>"},"type":" Synth"},"Gibberish.Synth2":{"text":"<h1 id=\"gibberishsynth2synth\">Gibberish.Synth2 </h1>\n\n<p>Oscillator + attack / decay envelope + 24db ladder filter. Basically the same as the <a href=\"javascript:displayDocs('Gibberish.Synth')\">Synth</a> object but with the addition of the filter. Note that the envelope controls both the amplitude of the oscillator and the cutoff frequency of the filter.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>Gibberish.init(); <br />\na = new Gibberish.Synth2({ attack:44, decay:44100, cutoff:.2, resonance:4 }).connect(); <br />\na.note(880); <br />\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{"note":"<h3 id=\"gibberishsynth2notemethod\">Gibberish.Synth2.note : method</h3>\n\n<p>Generate an enveloped note at the provided frequency  </p>\n\n<p>param <strong>frequency</strong> Number. The frequency for the oscillator. <br />\nparam <strong>amp</strong> Number. Optional. The volume to use.  </p>"},"properties":{"frequency":"<h3 id=\"gibberishsynth2frequencyproperty\">Gibberish.Synth2.frequency : property</h3>\n\n<p>Number. The frequency for the carrier oscillator. This is normally set using the note method but can also be modulated.</p>","pulsewidth":"<h3 id=\"gibberishsynth2pulsewidthproperty\">Gibberish.Synth2.pulsewidth : property</h3>\n\n<p>Number. The duty cycle for PWM synthesis</p>","attack":"<h3 id=\"gibberishsynth2attackproperty\">Gibberish.Synth2.attack : property</h3>\n\n<p>Number. The length of the attack portion of the envelope in samples. Note that the synth's envelope affects both amplitude and the index of the synth.</p>","decay":"<h3 id=\"gibberishsynth2decayproperty\">Gibberish.Synth2.decay : property</h3>\n\n<p>Number. The length of the decay portion of the envelope in samples. Note that the synth's envelope affects both amplitude and the index of the synth.</p>","cutoff":"<h3 id=\"gibberishsynth2cutoffproperty\">Gibberish.Synth2.cutoff : property</h3>\n\n<p>Number. 0..1. The cutoff frequency for the synth's filter.</p>","resonance":"<h3 id=\"gibberishsynth2resonanceproperty\">Gibberish.Synth2.resonance : property</h3>\n\n<p>Number. 0..50. Values above 4.5 are likely to produce shrieking feedback. You are warned.</p>","useLowPassFilter":"<h3 id=\"gibberishsynth2uselowpassfilterproperty\">Gibberish.Synth2.useLowPassFilter : property</h3>\n\n<p>Boolean. Default true. Whether to use a high-pass or low-pass filter.</p>","glide":"<h3 id=\"gibberishsynth2glideproperty\">Gibberish.Synth2.glide : property</h3>\n\n<p>Number. The synth has a one-pole filter attached to the carrier frequency. Set glide to a value between .999 and 1 to get pitch sweep between notes.</p>","amp":"<h3 id=\"gibberishsynth2ampproperty\">Gibberish.Synth2.amp : property</h3>\n\n<p>Number. The relative amplitude level of the synth.</p>","channels":"<h3 id=\"gibberishsynth2channelsproperty\">Gibberish.Synth2.channels : property</h3>\n\n<p>Number. Default 2. Mono or Stereo synthesis.</p>","pan":"<h3 id=\"gibberishsynth2panproperty\">Gibberish.Synth2.pan : property</h3>\n\n<p>Number. Default 0. If the synth has two channels, this determines its position in the stereo spectrum.</p>","waveform":"<h3 id=\"gibberishsynth2waveformproperty\">Gibberish.Synth2.waveform : property</h3>\n\n<p>String. The type of waveform to use. Options include 'Sine', 'Triangle', 'PWM', 'Saw' etc.</p>"},"type":" Synth"},"Gibberish.PolySynth2":{"text":"<h1 id=\"gibberishpolysynth2synth\">Gibberish.PolySynth2 </h1>\n\n<p>A polyphonic version of <a href=\"javascript:displayDocs('Gibberish.Synth2')). There are two additional properties for the polyphonic version of the synth. The polyphonic version consists of multiple Synths being fed into a single [Bus](javascript:displayDocs('Gibberish.Bus')\">Synth2</a> object.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>Gibberish.init(); <br />\na = new Gibberish.PolySynth2({ attack:88200, decay:88200, maxVoices:10 }).connect(); <br />\na.note(880); <br />\na.note(1320); \na.note(1760); <br />\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p>One important property to pass to the constructor is the maxVoices property, which defaults to 5. This controls how many voices are allocated to the synth and cannot be changed after initialization.  </p>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{"note":"<h3 id=\"gibberishpolysynth2notemethod\">Gibberish.PolySynth2.note : method</h3>\n\n<p>Generate an enveloped note at the provided frequency using a simple voice allocation system where if all children are active, the one active the longest cancels its current note and begins playing a new one.    </p>\n\n<p>param <strong>frequency</strong> Number. The frequency for the oscillator. \nparam <strong>amp</strong> Number. Optional. The volume to use.  </p>"},"properties":{"children":"<h3 id=\"gibberishpolysynth2childrenproperty\">Gibberish.PolySynth2.children : property</h3>\n\n<p>Array. Read-only. An array holding all of the child FMSynth objects.</p>","maxVoices":"<h3 id=\"gibberishpolysynth2maxvoicesproperty\">Gibberish.PolySynth2.maxVoices : property</h3>\n\n<p>Number. The number of voices of polyphony the synth has. May only be set in initialization properties passed to constrcutor.</p>"},"type":" Synth"},"Gibberish.FMSynth":{"text":"<h1 id=\"gibberishfmsynthsynth\">Gibberish.FMSynth </h1>\n\n<p>Classic 2-op FM synthesis with an attached attack / decay envelope.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>Gibberish.init(); <br />\na = new Gibberish.FMSynth({ cmRatio:5, index:3 }).connect();\na.note(880);</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{"note":"<h3 id=\"gibberishfmsynthnotemethod\">Gibberish.FMSynth.note : method</h3>\n\n<p>Generate an enveloped note at the provided frequency  </p>\n\n<p>param <strong>frequency</strong> Number. The frequency for the carrier oscillator. The modulator frequency will be calculated automatically from this value in conjunction with the synth's carrier to modulation ratio <br />\nparam <strong>amp</strong> Number. Optional. The volume to use.  </p>"},"properties":{"frequency":"<h3 id=\"gibberishfmsynthfrequencyproperty\">Gibberish.FMSynth.frequency : property</h3>\n\n<p>Number. The frequency for the carrier oscillator. This is normally set using the note method but can also be modulated.</p>","cmRatio":"<h3 id=\"gibberishfmsynthcmratioproperty\">Gibberish.FMSynth.cmRatio : property</h3>\n\n<p>Number. The carrier-to-modulation ratio. A cmRatio of 2 means that the carrier frequency will be twice the frequency of the modulator.</p>","attack":"<h3 id=\"gibberishfmsynthattackproperty\">Gibberish.FMSynth.attack : property</h3>\n\n<p>Number. The length of the attack portion of the envelope in samples. Note that the synth's envelope affects both amplitude and the index of the synth.</p>","decay":"<h3 id=\"gibberishfmsynthdecayproperty\">Gibberish.FMSynth.decay : property</h3>\n\n<p>Number. The length of the decay portion of the envelope in samples. Note that the synth's envelope affects both amplitude and the index of the synth.</p>","glide":"<h3 id=\"gibberishfmsynthglideproperty\">Gibberish.FMSynth.glide : property</h3>\n\n<p>Number. The synth has a one-pole filter attached to the carrier frequency. Set glide to a value between .999 and 1 to get pitch sweep between notes.</p>","amp":"<h3 id=\"gibberishfmsynthampproperty\">Gibberish.FMSynth.amp : property</h3>\n\n<p>Number. The relative amplitude level of the synth.</p>","channels":"<h3 id=\"gibberishfmsynthchannelsproperty\">Gibberish.FMSynth.channels : property</h3>\n\n<p>Number. Default 2. Mono or Stereo synthesis.</p>","pan":"<h3 id=\"gibberishfmsynthpanproperty\">Gibberish.FMSynth.pan : property</h3>\n\n<p>Number. Default 0. If the synth has two channels, this determines its position in the stereo spectrum.</p>"},"type":" Synth"},"Gibberish.PolyFM":{"text":"<h1 id=\"gibberishpolyfmsynth\">Gibberish.PolyFM </h1>\n\n<p>A polyphonic version of <a href=\"javascript:displayDocs('Gibberish.FMSynth')). There are two additional properties for the polyphonic version of the synth. The polyphonic version consists of multiple FMSynths being fed into a single [Bus](javascript:displayDocs('Gibberish.Bus')\">FMSynth</a> object.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>Gibberish.init(); <br />\na = new Gibberish.PolyFM({ cmRatio:5, index:3, attack:88200, decay:88200 }).connect(); <br />\na.note(880); <br />\na.note(1320); <br />\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p>One important property to pass to the constructor is the maxVoices property, which defaults to 5. This controls how many voices are allocated to the synth and cannot be changed after initialization.  </p>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{"note":"<h3 id=\"gibberishpolyfmnotemethod\">Gibberish.PolyFM.note : method</h3>\n\n<p>Generate an enveloped note at the provided frequency using a simple voice allocation system where if all children are active, the one active the longest cancels its current note and begins playing a new one.    </p>\n\n<p>param <strong>frequency</strong> Number. The frequency for the carrier oscillator. The modulator frequency will be calculated automatically from this value in conjunction with the synth's <br />\nparam <strong>amp</strong> Number. Optional. The volume to use.  </p>"},"properties":{"children":"<h3 id=\"gibberishpolyfmchildrenproperty\">Gibberish.PolyFM.children : property</h3>\n\n<p>Array. Read-only. An array holding all of the child FMSynth objects.</p>","maxVoices":"<h3 id=\"gibberishpolyfmmaxvoicesproperty\">Gibberish.PolyFM.maxVoices : property</h3>\n\n<p>Number. The number of voices of polyphony the synth has. May only be set in initialization properties passed to constrcutor.</p>"},"type":" Synth"},"Gibberish.MonoSynth":{"text":"<h1 id=\"gibberishmonosynthsynth\">Gibberish.MonoSynth </h1>\n\n<p>A three oscillator monosynth for bass and lead lines. You can set the octave and tuning offsets for oscillators 2 &amp; 3. There is a 24db filter and an envelope controlling\nboth the amplitude and filter cutoff.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>\nt = new Gibberish.Mono({ <br />\n    cutoff:0, <br />\n    filterMult:.5, <br />\n    attack:_8, <br />\n    decay:_8, <br />\n    octave2:-1, <br />\n    octave3:-1, <br />\n    detune2:.01, <br />\n    glide:_12, <br />\n}).connect(); <br />\nt.note(\"C3\");</code></p>\n\n<h2 id=\"constructors\">Constructors</h2>\n\n<p>param <strong>arguments</strong> : Object. A dictionary of property values to set upon initialization. See the properties section and the example usage section for details.</p>","methods":{"note":"<h3 id=\"gibberishmonosynthnotemethod\">Gibberish.MonoSynth.note : method</h3>\n\n<p>param <strong>note or frequency</strong> : String or Integer. You can pass a note name, such as \"A#4\", or a frequency value, such as 440.\nparam <strong>amp</strong> : Optional. Float. The volume of the note, usually between 0..1. The main amp property of the Synth will also affect note amplitude.</p>"},"properties":{"waveform":"<h3 id=\"gibberishmonosynthwaveformproperty\">Gibberish.MonoSynth.waveform : property</h3>\n\n<p>String. The primary oscillator to be used. Can currently be 'Sine', 'Square', 'Noise', 'Triangle' or 'Saw'. </p>","attack":"<h3 id=\"gibberishmonosynthattackproperty\">Gibberish.MonoSynth.attack : property</h3>\n\n<p>Integer. The length, in samples, of the attack of the amplitude envelope.</p>","decay":"<h3 id=\"gibberishmonosynthdecayproperty\">Gibberish.MonoSynth.decay : property</h3>\n\n<p>Integer. The length, in samples, of the decay of the amplitude envelope.</p>","amp":"<h3 id=\"gibberishmonosynthampproperty\">Gibberish.MonoSynth.amp : property</h3>\n\n<p>Float. The peak amplitude of the synth, usually between 0..1</p>","cutoff":"<h3 id=\"gibberishmonosynthcutoffproperty\">Gibberish.MonoSynth.cutoff : property</h3>\n\n<p>Float. The frequency cutoff for the synth's filter. Range is 0..1.</p>","filterMult":"<h3 id=\"gibberishmonosynthfiltermultproperty\">Gibberish.MonoSynth.filterMult : property</h3>\n\n<p>Float. As the envelope on the synth progress, the filter cutoff will also change by this amount * the envelope amount.</p>","resonance":"<h3 id=\"gibberishmonosynthresonanceproperty\">Gibberish.MonoSynth.resonance : property</h3>\n\n<p>Float. The emphasis placed on the filters cutoff frequency. 0..50, however, GOING OVER 5 IS DANGEROUS TO YOUR EARS (ok, maybe 6 is all right...)</p>","octave2":"<h3 id=\"gibberishmonosynthoctave2property\">Gibberish.MonoSynth.octave2 : property</h3>\n\n<p>Integer. The octave difference between oscillator 1 and oscillator 2. Can be positive (higher osc2) or negative (lower osc 2) or 0 (same octave).</p>","detune2":"<h3 id=\"gibberishmonosynthdetune2property\">Gibberish.MonoSynth.detune2 : property</h3>\n\n<p>Float. The amount, from -1..1, the oscillator 2 is detuned. A value of -.5 means osc2 is half an octave lower than osc1. A value of .01 means osc2 is .01 octaves higher than osc1.</p>","octave3":"<h3 id=\"gibberishmonosynthoctave3property\">Gibberish.MonoSynth.octave3 : property</h3>\n\n<p>Integer. The octave difference between oscillator 1 and oscillator 3. Can be positive (higher osc3) or negative (lower osc 3) or 0 (same octave).</p>","detune3":"<h3 id=\"gibberishmonosynthdetune3property\">Gibberish.MonoSynth.detune3 : property</h3>\n\n<p>Float. The amount, from -1..1, the oscillator 3 is detuned. A value of -.5 means osc3 is half an octave lower than osc1. A value of .01 means osc3 is .01 octaves higher than osc1.</p>","glide":"<h3 id=\"gibberishmonosynthglideproperty\">Gibberish.MonoSynth.glide : property</h3>\n\n<p>Integer. The length in time, in samples, to slide in pitch from one note to the next.</p>"},"type":" Synth"},"Gibberish.Distortion":{"text":"<h1 id=\"gibberishdistortionfx\">Gibberish.Distortion </h1>\n\n<p>A simple waveshaping distortion that adaptively scales its gain based on the amount of distortion applied.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>a = new Gibberish.Synth({ attack:44, decay:44100 }); <br />\nb = new Gibberish.Distortion({ input:a, amount:30 }).connect(); <br />\na.note(440); <br />\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{},"properties":{"amount":"<h3 id=\"gibberishdistortionamountproperty\">Gibberish.Distortion.amount : property</h3>\n\n<p>Number. The amount of distortion to apply. This number cannot be set lower than 2.</p>"},"type":" FX"},"Gibberish.Delay":{"text":"<h1 id=\"gibberishdelayfx\">Gibberish.Delay </h1>\n\n<p>A simple echo effect.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>a = new Gibberish.Synth({ attack:44, decay:44100 }); <br />\nb = new Gibberish.Delay({ input:a, time:22050, feedback:.35 }).connect(); <br />\na.note(440); <br />\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{},"properties":{"time":"<h3 id=\"gibberishdelaytimeproperty\">Gibberish.Delay.time : property</h3>\n\n<p>Number. The delay time as measured in samples</p>","feedback":"<h3 id=\"gibberishdelayfeedbackproperty\">Gibberish.Delay.feedback : property</h3>\n\n<p>Number. The amount of feedback that the delay puts into its buffers.</p>"},"type":" FX"},"Gibberish.Decimator":{"text":"<h1 id=\"gibberishdecimatorfx\">Gibberish.Decimator </h1>\n\n<p>A bit-crusher / sample rate reducer. Adapted from code / comments at http://musicdsp.org/showArchiveComment.php?ArchiveID=124</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>a = new Gibberish.Synth({ attack:44, decay:44100 }); <br />\nb = new Gibberish.Decimator({ input:a, bitDepth:4.2, sampleRate:.33 }).connect(); <br />\na.note(440); <br />\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{},"properties":{"bitDepth":"<h3 id=\"gibberishdecimatorbitdepthproperty\">Gibberish.Decimator.bitDepth : property</h3>\n\n<p>Float. 0..16. The number of bits the signal is truncated to. May be a floating point number.</p>","sampleRate":"<h3 id=\"gibberishdecimatorsamplerateproperty\">Gibberish.Decimator.sampleRate : property</h3>\n\n<p>Number. 0..1. The sample rate to use where 0 is 0 Hz and 1 is nyquist.</p>"},"type":" FX"},"Gibberish.RingModulation":{"text":"<h1 id=\"gibberishringmodulationfx\">Gibberish.RingModulation </h1>\n\n<p>The name says it all. This ugen also has a mix property to control the ratio of wet to dry output.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>a = new Gibberish.Synth({ attack:44, decay:44100 }); <br />\nb = new Gibberish.RingModulation({ input:a, frequency:1000, amp:.4, mix:1 }).connect(); <br />\na.note(440); <br />\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{},"properties":{"frequency":"<h3 id=\"gibberishringmodulationfrequencyproperty\">Gibberish.RingModulation.frequency : property</h3>\n\n<p>Float. The frequency of the ring modulation modulator wave.</p>","amp":"<h3 id=\"gibberishringmodulationampproperty\">Gibberish.RingModulation.amp : property</h3>\n\n<p>Float. The amplitude of the ring modulation modulator wave.</p>","mix":"<h3 id=\"gibberishringmodulationmixproperty\">Gibberish.RingModulation.mix : property</h3>\n\n<p>Float. 0..1. The wet/dry output ratio. A value of 1 means a completely wet signal, a value of 0 means completely dry.</p>"},"type":" FX"},"Gibberish.OnePole":{"text":"<h1 id=\"gibberishonepolefx\">Gibberish.OnePole </h1>\n\n<p>A one-pole filter for smoothing property values. This is particularly useful when the properties are being controlled interactively. You use the smooth method to apply the filter.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>a = new Gibberish.Synth({ attack:44, decay:44100 }).connect(); <br />\nb = new Gibberish.OnePole({input:a.properties.frequency, a0:.0001, b1:.9999}); <br />\nb.smooth('frequency', a); <br />\na.note(880); <br />\na.note(440); <br />\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{"smooth":"<h3 id=\"gibberishonepolesmoothmethod\">Gibberish.OnePole.smooth : method</h3>\n\n<p>Use this to apply the filter to a property of an object.</p>\n\n<p>param <strong>propertyName</strong> String. The name of the property to smooth. <br />\nparam <strong>object</strong> Object. The object containing the property to be smoothed</p>","remove":"<h3 id=\"gibberishonepoleremovemethod\">Gibberish.OnePole.remove : method</h3>\n\n<p>Remove OnePole from assigned ugen property. This will effectively remove the filter from the graph and return the normal target ugen property behavior.</p>"},"properties":{"input":"<h3 id=\"gibberishonepoleinputproperty\">Gibberish.OnePole.input : property</h3>\n\n<p>Float. The property to smooth. You should always refer to this property through the properties dictionary of the ugen. In general it is much easier to use the smooth method of the OnePole than to set this property manually.</p>","a0":"<h3 id=\"gibberishonepolea0property\">Gibberish.OnePole.a0 : property</h3>\n\n<p>Float. The value the input is multiplied by.</p>","b1":"<h3 id=\"gibberishonepoleb1property\">Gibberish.OnePole.b1 : property</h3>\n\n<p>Float. The value this pole of the filter is multiplied by.</p>"},"type":" FX"},"Gibberish.Filter24":{"text":"<h1 id=\"gibberishfilter24fx\">Gibberish.Filter24 </h1>\n\n<p>A four pole ladder filter. Adapted from Arif Ove Karlsne's 24dB ladder approximation: http://musicdsp.org/showArchiveComment.php?ArchiveID=141.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>a = new Gibberish.Synth({ attack:44, decay:44100 }); <br />\nb = new Gibberish.Filter24({input:a, cutoff:.2, resonance:4}).connect(); <br />\na.note(1760); <br />\na.note(440); <br />\na.isLowPass = false; <br />\na.note(220); <br />\na.note(1760); <br />\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{},"properties":{"input":"<h3 id=\"gibberishfilter24inputproperty\">Gibberish.Filter24.input : property</h3>\n\n<p>Object. The ugen that should feed the filter.</p>","cutoff":"<h3 id=\"gibberishfilter24cutoffproperty\">Gibberish.Filter24.cutoff : property</h3>\n\n<p>Number. 0..1. The cutoff frequency for the synth's filter.</p>","resonance":"<h3 id=\"gibberishfilter24resonanceproperty\">Gibberish.Filter24.resonance : property</h3>\n\n<p>Number. 0..50. Values above 4.5 are likely to produce shrieking feedback. You are warned.</p>","isLowPass":"<h3 id=\"gibberishfilter24islowpassproperty\">Gibberish.Filter24.isLowPass : property</h3>\n\n<p>Boolean. Default true. Whether to use a low-pass or high-pass filter.</p>"},"type":" FX"},"Gibberish.SVF":{"text":"<h1 id=\"gibberishsvffx\">Gibberish.SVF </h1>\n\n<p>A two-pole state variable filter. This filter calculates coefficients on a per-sample basis, so that you can easily modulate cutoff and Q. Can switch between low-pass, high-pass, band and notch modes.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>a = new Gibberish.Synth({ attack:44, decay:44100 }); <br />\nb = new Gibberish.SVF({input:a, cutoff:200, Q:4, mode:0}); <br />\na.note(1760); <br />\na.note(440); <br />\na.mode = 2;\na.note(220); <br />\na.note(1760); <br />\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{},"properties":{"input":"<h3 id=\"gibberishsvfinputproperty\">Gibberish.SVF.input : property</h3>\n\n<p>Object. The ugen that should feed the filter.</p>","cutoff":"<h3 id=\"gibberishsvfcutoffproperty\">Gibberish.SVF.cutoff : property</h3>\n\n<p>Number. 0..22050. The cutoff frequency for the synth's filter. Note that unlike the Filter24, this is measured in Hz.</p>","resonance":"<h3 id=\"gibberishsvfresonanceproperty\">Gibberish.SVF.resonance : property</h3>\n\n<p>Number. 0..50. Values above 4.5 are likely to produce shrieking feedback. You are warned.</p>","mode":"<h3 id=\"gibberishsvfmodeproperty\">Gibberish.SVF.mode : property</h3>\n\n<p>Number. 0..3. 0 = lowpass, 1 = highpass, 2 = bandpass, 3 = notch.</p>"},"type":" FX"},"Gibberish.Biquad":{"text":"<h1 id=\"gibberishbiquadfx\">Gibberish.Biquad </h1>\n\n<p>A two-pole biquad filter. Currently, you must manually call calculateCoefficients every time mode, cutoff or Q changes; thus this filter isn't good for samplerate modulation.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>a = new Gibberish.Synth({ attack:44, decay:44100 }); <br />\nb = new Gibberish.Biquad({input:a, cutoff:200, Q:4, mode:\"LP\"}).connect(); <br />\na.note(1760); <br />\na.note(440); <br />\na.mode = \"HP\";\na.note(220); <br />\na.note(1760); <br />\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{},"properties":{"input":"<h3 id=\"gibberishbiquadinputproperty\">Gibberish.Biquad.input : property</h3>\n\n<p>Object. The ugen that should feed the filter.</p>","cutoff":"<h3 id=\"gibberishbiquadcutoffproperty\">Gibberish.Biquad.cutoff : property</h3>\n\n<p>Number. 0..22050. The cutoff frequency for the synth's filter. Note that unlike the Filter24, this is measured in Hz.</p>","Q":"<h3 id=\"gibberishbiquadqproperty\">Gibberish.Biquad.Q : property</h3>\n\n<p>Number. 0..50. Values above 4.5 are likely to produce shrieking feedback. You are warned.</p>","mode":"<h3 id=\"gibberishbiquadmodeproperty\">Gibberish.Biquad.mode : property</h3>\n\n<p>Number. 0..3. \"LP\" = lowpass, \"HP\" = highpass, \"BP\" = bandpass</p>"},"type":" FX"},"Gibberish.Flanger":{"text":"<h1 id=\"gibberishflangerfx\">Gibberish.Flanger </h1>\n\n<p>Classic flanging effect with feedback.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>a = new Gibberish.Synth({ attack:44, decay:44100 }); <br />\nb = new Gibberish.Flanger({input:a, rate:.5, amount:125, feedback:.5}).connect(); <br />\na.note(440); <br />\na.feedback = 0; <br />\na.note(440); <br />\na.rate = 4;\na.note(440);\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{},"properties":{"input":"<h3 id=\"gibberishflangerinputproperty\">Gibberish.Flanger.input : property</h3>\n\n<p>Object. The ugen that should feed the flagner.</p>","rate":"<h3 id=\"gibberishflangerrateproperty\">Gibberish.Flanger.rate : property</h3>\n\n<p>Number. The speed at which the delay line tap position is modulated.</p>","amount":"<h3 id=\"gibberishflangeramountproperty\">Gibberish.Flanger.amount : property</h3>\n\n<p>Number. The amount of time, in samples, that the delay line tap position varies by.</p>","feedback":"<h3 id=\"gibberishflangerfeedbackproperty\">Gibberish.Flanger.feedback : property</h3>\n\n<p>Number. The amount of output that should be fed back into the delay line</p>","offset":"<h3 id=\"gibberishflangeroffsetproperty\">Gibberish.Flanger.offset : property</h3>\n\n<p>Number. The base offset of the delay line tap from the current time. Large values (> 500) lead to chorusing effects.</p>"},"type":" FX"},"Gibberish.Vibrato":{"text":"<h1 id=\"gibberishvibratofx\">Gibberish.Vibrato </h1>\n\n<p>Delay line vibrato effect.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>a = new Gibberish.Synth({ attack:44, decay:44100 }); <br />\nb = new Gibberish.Vibrato({input:a, rate:4, amount:125 }).connect(); <br />\na.note(440); <br />\na.rate = .5;\na.note(440);\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the synth on initialization.</p>\n\n<hr />","methods":{},"properties":{"input":"<h3 id=\"gibberishvibratoinputproperty\">Gibberish.Vibrato.input : property</h3>\n\n<p>Object. The ugen that should feed the vibrato.</p>","rate":"<h3 id=\"gibberishvibratorateproperty\">Gibberish.Vibrato.rate : property</h3>\n\n<p>Number. The speed at which the delay line tap position is modulated.</p>","amount":"<h3 id=\"gibberishvibratoamountproperty\">Gibberish.Vibrato.amount : property</h3>\n\n<p>Number. The size of the delay line modulation; effectively the amount of vibrato to produce, </p>","offset":"<h3 id=\"gibberishvibratooffsetproperty\">Gibberish.Vibrato.offset : property</h3>\n\n<p>Number. The base offset of the delay line tap from the current time.</p>"},"type":" FX"},"Gibberish.BufferShuffler":{"text":"<h1 id=\"gibberishbuffershufflerfx\">Gibberish.BufferShuffler </h1>\n\n<p>A buffer shuffling / stuttering effect with reversing and pitch-shifting</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>a = new Gibberish.Synth({ attack:88200, decay:88200 }); <br />\nb = new Gibberish.BufferShuffler({input:a, chance:.25, amount:125, rate:44100, pitchMin:-4, pitchMax:4 }).connect(); <br />\na.note(440);\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em> : Object. A dictionary of property keys and values to assign to the Gibberish.BufferShuffler object</p>\n\n<hr />","methods":{},"properties":{"chance":"<h3 id=\"gibberishbuffershufflerchanceproperty\">Gibberish.BufferShuffler.chance : property</h3>\n\n<p>Float. Range 0..1. Default .25. The likelihood that incoming audio will be shuffled.</p>","rate":"<h3 id=\"gibberishbuffershufflerrateproperty\">Gibberish.BufferShuffler.rate : property</h3>\n\n<p>Integer, in samples. Default 11025. How often Gibberish.BufferShuffler will randomly decide whether or not to shuffle.</p>","length":"<h3 id=\"gibberishbuffershufflerlengthproperty\">Gibberish.BufferShuffler.length : property</h3>\n\n<p>Integer, in samples. Default 22050. The length of time to play stuttered audio when stuttering occurs.</p>","reverseChance":"<h3 id=\"gibberishbuffershufflerreversechanceproperty\">Gibberish.BufferShuffler.reverseChance : property</h3>\n\n<p>Float. Range 0..1. Default .5. The likelihood that stuttered audio will be reversed</p>","pitchChance":"<h3 id=\"gibberishbuffershufflerpitchchanceproperty\">Gibberish.BufferShuffler.pitchChance : property</h3>\n\n<p>Float. Range 0..1. Default .5. The likelihood that stuttered audio will be repitched.</p>","pitchMin":"<h3 id=\"gibberishbuffershufflerpitchminproperty\">Gibberish.BufferShuffler.pitchMin : property</h3>\n\n<p>Float. Range 0..1. Default .25. The lowest playback speed used to repitch the audio</p>","pitchMax":"<h3 id=\"gibberishbuffershufflerpitchmaxproperty\">Gibberish.BufferShuffler.pitchMax : property</h3>\n\n<p>Float. Range 0..1. Default 2. The highest playback speed used to repitch the audio.</p>","wet":"<h3 id=\"gibberishbuffershufflerwetproperty\">Gibberish.BufferShuffler.wet : property</h3>\n\n<p>Float. Range 0..1. Default 1. When shuffling, the amplitude of the wet signal</p>","dry":"<h3 id=\"gibberishbuffershufflerdryproperty\">Gibberish.BufferShuffler.dry : property</h3>\n\n<p>Float. Range 0..1. Default 0. When shuffling, the amplitude of the dry signal</p>"},"type":" FX"},"Gibberish.Reverb":{"text":"<h1 id=\"gibberishreverbfx\">Gibberish.Reverb </h1>\n\n<p>based off audiolib.js reverb and freeverb</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>a = new Gibberish.Synth({ attack:88200, decay:88200 }); <br />\nb = new Gibberish.Reverb({input:a, roomSize:.5, wet:1, dry;.25}).connect();\na.note(440);\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em> : Object. A dictionary of property keys and values to assign to the Gibberish.BufferShuffler object</p>","methods":{},"properties":{"roomSize":"<h3 id=\"gibberishreverbroomsizeproperty\">Gibberish.Reverb.roomSize : property</h3>\n\n<p>Float. 0..1. The size of the room being emulated.</p>","damping":"<h3 id=\"gibberishreverbdampingproperty\">Gibberish.Reverb.damping : property</h3>\n\n<p>Float. Attenuation of high frequencies that occurs.</p>","wet":"<h3 id=\"gibberishreverbwetproperty\">Gibberish.Reverb.wet : property</h3>\n\n<p>Float. Default = .75. The amount of processed signal that is output.  </p>","dry":"<h3 id=\"gibberishreverbdryproperty\">Gibberish.Reverb.dry : property</h3>\n\n<p>Float. Default = .5. The amount of dry signal that is output</p>"},"type":" FX"},"Gibberish.Granulator":{"text":"<h1 id=\"gibberishgranulatorfx\">Gibberish.Granulator </h1>\n\n<p>A granulator that operates on a buffer of samples. You can get the samples from a <a href=\"javascript:displayDocs('Gibberish.Sampler')\">Sampler</a>\nobject.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>a = new Gibberish.Sampler('resources/trumpet.wav'); <br />\n// wait until sample is loaded to create granulator <br />\na.onload = function() { <br />\n  b = new Gibberish.Granulator({ <br />\n    buffer:a.getBuffer(), <br />\n    grainSize:1000, <br />\n    speedMin: -2, <br />\n    speedMax: 2, <br />\n  }); <br />\n  b.mod('position', new Gibberish.Sine(.1, .45), '+'); <br />\n  b.connect(); <br />\n};</code></p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>propertiesList</em>: Object. At a minimum you should define the input to granulate. See the example.</p>","methods":{},"properties":{"speed":"<h3 id=\"gibberishgranulatorspeedproperty\">Gibberish.Granulator.speed : property</h3>\n\n<p>Float. The playback rate, in samples, of each grain</p>","speedMin":"<h3 id=\"gibberishgranulatorspeedminproperty\">Gibberish.Granulator.speedMin : property</h3>\n\n<p>Float. When set, the playback rate will vary on a per grain basis from (grain.speed + grain.speedMin) -> (grain.speed + grain.speedMax). This value should almost always be negative.</p>","speedMax":"<h3 id=\"gibberishgranulatorspeedmaxproperty\">Gibberish.Granulator.speedMax : property</h3>\n\n<p>Float. When set, the playback rate will vary on a per grain basis from (grain.speed + grain.speedMin) -> (grain.speed + grain.speedMax).</p>","grainSize":"<h3 id=\"gibberishgranulatorgrainsizeproperty\">Gibberish.Granulator.grainSize : property</h3>\n\n<p>Integer. The length, in samples, of each grain</p>","position":"<h3 id=\"gibberishgranulatorpositionproperty\">Gibberish.Granulator.position : property</h3>\n\n<p>Float. The center position of the grain cloud. 0 represents the start of the buffer, 1 represents the end.</p>","positionMin":"<h3 id=\"gibberishgranulatorpositionminproperty\">Gibberish.Granulator.positionMin : property</h3>\n\n<p>Float. The left boundary on the time axis of the grain cloud.</p>","positionMax":"<h3 id=\"gibberishgranulatorpositionmaxproperty\">Gibberish.Granulator.positionMax : property</h3>\n\n<p>Float. The right boundary on the time axis of the grain cloud.</p>","buffer":"<h3 id=\"gibberishgranulatorbufferproperty\">Gibberish.Granulator.buffer : property</h3>\n\n<p>Object. The input buffer to granulate.</p>","numberOfGrains":"<h3 id=\"gibberishgranulatornumberofgrainsproperty\">Gibberish.Granulator.numberOfGrains : property</h3>\n\n<p>Float. The number of grains in the cloud. Can currently only be set on initialization.</p>"},"type":" FX"},"Gibberish.Sine":{"text":"<h1 id=\"gibberishsineoscillator\">Gibberish.Sine </h1>\n\n<p>A sinewave calculated on a per-sample basis.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>// make a sine wave <br />\nGibberish.init(); <br />\na = new Gibberish.Sine().connect();</code></p>\n\n<hr />","methods":{},"properties":{"frequency":"<h3 id=\"gibberishsinefrequencyproperty\">Gibberish.Sine.frequency : property</h3>\n\n<p>Number. From 20 - 20000 hz.</p>","amp":"<h3 id=\"gibberishsineampproperty\">Gibberish.Sine.amp : property</h3>\n\n<p>Number. A linear value specifying relative amplitude, ostensibly from 0..1 but can be higher, or lower when used for modulation.</p>"},"type":" Oscillator"},"Gibberish.Sine2":{"text":"<h1 id=\"gibberishsine2oscillator\">Gibberish.Sine2 </h1>\n\n<p>A sinewave calculated on a per-sample basis that can be panned.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>// make a sine wave <br />\nGibberish.init(); <br />\na = new Gibberish.Sine2(880, .5, -.25).connect();</code></p>\n\n<hr />","methods":{"callback":"<h3 id=\"gibberishsine2callbackmethod\">Gibberish.Sine2.callback : method</h3>\n\n<p>Returns a stereo sample of output as an array.  </p>\n\n<p>param <strong>frequency</strong> Number. The frequency to be used to calculate output. <br />\nparam <strong>amp</strong> Number. The amplitude to be used to calculate output. <br />\nparam <strong>pan</strong> Number. The position in the stereo spectrum of the signal.</p>"},"properties":{"frequency":"<h3 id=\"gibberishsine2frequencyproperty\">Gibberish.Sine2.frequency : property</h3>\n\n<p>Number. From 20 - 20000 hz.</p>","amp":"<h3 id=\"gibberishsine2ampproperty\">Gibberish.Sine2.amp : property</h3>\n\n<p>Number. A linear value specifying relative amplitude, ostensibly from 0..1 but can be higher, or lower when used for modulation.</p>","pan":"<h3 id=\"gibberishsine2panproperty\">Gibberish.Sine2.pan : property</h3>\n\n<p>Number. -1..1. The position of the sinewave in the stereo spectrum</p>"},"type":" Oscillator"},"Gibberish.Saw":{"text":"<h1 id=\"gibberishsawoscillator\">Gibberish.Saw </h1>\n\n<p>A non-bandlimited saw wave calculated on a per-sample basis.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>// make a saw wave <br />\nGibberish.init(); <br />\na = new Gibberish.Saw(330, .4).connect();</code></p>\n\n<hr />","methods":{},"properties":{"frequency":"<h3 id=\"gibberishsawfrequencyproperty\">Gibberish.Saw.frequency : property</h3>\n\n<p>Number. From 20 - 20000 hz.</p>","amp":"<h3 id=\"gibberishsawampproperty\">Gibberish.Saw.amp : property</h3>\n\n<p>Number. A linear value specifying relative amplitude, ostensibly from 0..1 but can be higher, or lower when used for modulation.</p>"},"type":" Oscillator"},"Gibberish.Saw2":{"text":"<h1 id=\"gibberishsaw2oscillator\">Gibberish.Saw2 </h1>\n\n<p>A stereo, non-bandlimited saw wave calculated on a per-sample basis.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>// make a saw wave <br />\nGibberish.init(); <br />\na = new Gibberish.Saw2(330, .4).connect();</code></p>\n\n<hr />","methods":{"callback":"<h3 id=\"gibberishsaw2callbackmethod\">Gibberish.Saw2.callback : method</h3>\n\n<p>Returns a stereo sample of output as an array.  </p>\n\n<p>param <strong>frequency</strong> Number. The frequency to be used to calculate output. <br />\nparam <strong>amp</strong> Number. The amplitude to be used to calculate output. <br />\nparam <strong>pan</strong> Number. The position in the stereo spectrum of the signal.</p>"},"properties":{},"type":" Oscillator"},"Gibberish.Triangle":{"text":"<h1 id=\"gibberishtriangleoscillator\">Gibberish.Triangle </h1>\n\n<p>A triangle calculated on a per-sample basis.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>// make a triangle wave <br />\nGibberish.init(); <br />\na = new Gibberish.Triangle({frequency:570, amp:.35}).connect();</code></p>\n\n<hr />","methods":{},"properties":{"frequency":"<h3 id=\"gibberishtrianglefrequencyproperty\">Gibberish.Triangle.frequency : property</h3>\n\n<p>Number. From 20 - 20000 hz.</p>","amp":"<h3 id=\"gibberishtriangleampproperty\">Gibberish.Triangle.amp : property</h3>\n\n<p>Number. A linear value specifying relative amplitude, ostensibly from 0..1 but can be higher, or lower when used for modulation.</p>"},"type":" Oscillator"},"Gibberish.Triangle2":{"text":"<h1 id=\"gibberishtriangle2oscillator\">Gibberish.Triangle2 </h1>\n\n<p>A triangle calculated on a per-sample basis that can be panned.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>Gibberish.init(); <br />\na = new Gibberish.Triangle2(880, .5, -.25).connect();</code></p>\n\n<hr />","methods":{"callback":"<h3 id=\"gibberishtriangle2callbackmethod\">Gibberish.Triangle2.callback : method</h3>\n\n<p>Returns a stereo sample of output as an array.  </p>\n\n<p>param <strong>frequency</strong> Number. The frequency to be used to calculate output. <br />\nparam <strong>amp</strong> Number. The amplitude to be used to calculate output. <br />\nparam <strong>pan</strong> Number. The position in the stereo spectrum of the signal.</p>"},"properties":{"frequency":"<h3 id=\"gibberishtriangle2frequencyproperty\">Gibberish.Triangle2.frequency : property</h3>\n\n<p>Number. From 20 - 20000 hz.</p>","amp":"<h3 id=\"gibberishtriangle2ampproperty\">Gibberish.Triangle2.amp : property</h3>\n\n<p>Number. A linear value specifying relative amplitude, ostensibly from 0..1 but can be higher, or lower when used for modulation.</p>","pan":"<h3 id=\"gibberishtriangle2panproperty\">Gibberish.Triangle2.pan : property</h3>\n\n<p>Number. -1..1. The position of the triangle wave in the stereo spectrum</p>"},"type":" Oscillator"},"Gibberish.Saw3":{"text":"<h1 id=\"gibberishsaw3oscillator\">Gibberish.Saw3 </h1>\n\n<p>A bandlimited saw wave created using FM feedback, see http://scp.web.elte.hu/papers/synthesis1.pdf.  </p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>// make a saw wave <br />\nGibberish.init(); <br />\na = new Gibberish.Saw3(330, .4).connect();</code></p>\n\n<hr />","methods":{"callback":"<h3 id=\"gibberishsaw3callbackmethod\">Gibberish.Saw3.callback : method</h3>\n\n<p>Returns a single sample of output.  </p>\n\n<p>param <strong>frequency</strong> Number. The frequency to be used to calculate output. <br />\nparam <strong>amp</strong> Number. The amplitude to be used to calculate output.  </p>"},"properties":{"frequency":"<h3 id=\"gibberishsaw3frequencyproperty\">Gibberish.Saw3.frequency : property</h3>\n\n<p>Number. From 20 - 20000 hz.</p>","amp":"<h3 id=\"gibberishsaw3ampproperty\">Gibberish.Saw3.amp : property</h3>\n\n<p>Number. A linear value specifying relative ampltiude, ostensibly from 0..1 but can be higher, or lower when used for modulation.</p>"},"type":" Oscillator"},"Gibberish.PWM":{"text":"<h1 id=\"gibberishpwmoscillator\">Gibberish.PWM </h1>\n\n<p>A bandlimited pulsewidth modulation wave created using FM feedback, see http://scp.web.elte.hu/papers/synthesis1.pdf.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>// make a pwm wave <br />\nGibberish.init(); <br />\na = new Gibberish.PWM(330, .4, .9).connect();</code></p>\n\n<hr />","methods":{"callback":"<h3 id=\"gibberishpwmcallbackmethod\">Gibberish.PWM.callback : method</h3>\n\n<p>Returns a single sample of output.  </p>\n\n<p>param <strong>frequency</strong> Number. The frequency to be used to calculate output. <br />\nparam <strong>amp</strong> Number. The amplitude to be used to calculate output. <br />\nparam <strong>pulsewidth</strong> Number. The duty cycle of the waveform</p>"},"properties":{"frequency":"<h3 id=\"gibberishpwmfrequencyproperty\">Gibberish.PWM.frequency : property</h3>\n\n<p>Number. From 20 - 20000 hz.</p>","amp":"<h3 id=\"gibberishpwmampproperty\">Gibberish.PWM.amp : property</h3>\n\n<p>Number. A linear value specifying relative ampltiude, ostensibly from 0..1 but can be higher, or lower when used for modulation.</p>","pulsewidth":"<h3 id=\"gibberishpwmpulsewidthproperty\">Gibberish.PWM.pulsewidth : property</h3>\n\n<p>Number. 0..1. The width of the waveform's duty cycle.</p>"},"type":" Oscillator"},"Gibberish.Noise":{"text":"<h1 id=\"gibberishnoiseoscillator\">Gibberish.Noise </h1>\n\n<p>A white noise oscillator</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>// make some noise\nGibberish.init(); <br />\na = new Gibberish.Noise(.4).connect();</code></p>\n\n<hr />","methods":{},"properties":{"amp":"<h3 id=\"gibberishnoiseampproperty\">Gibberish.Noise.amp : property</h3>\n\n<p>Number. A linear value specifying relative amplitude, ostensibly from 0..1 but can be higher, or lower when used for modulation.</p>"},"type":" Oscillator"},"Gibberish.KarplusStrong":{"text":"<h1 id=\"gibberishkarplusstrongphysicalmodel\">Gibberish.KarplusStrong </h1>\n\n<p>A plucked-string model.  </p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>Gibberish.init(); <br />\na = new Gibberish.KarplusStrong({ damping:.6 }).connect(); <br />\na.note(440);\n</code></p>\n\n<hr />","methods":{},"properties":{"blend":"<h3 id=\"gibberishkarplusstrongblendproperty\">Gibberish.KarplusStrong.blend : property</h3>\n\n<p>Number. 0..1. The likelihood that the sign of any given sample will be flipped. A value of 1 means there is no chance, a value of 0 means each samples sign will be flipped. This introduces noise into the model which can be used for various effects.</p>","damping":"<h3 id=\"gibberishkarplusstrongdampingproperty\">Gibberish.KarplusStrong.damping : property</h3>\n\n<p>Number. 0..1. Higher amounts of damping shorten the decay of the sound generated by each note.</p>","amp":"<h3 id=\"gibberishkarplusstrongampproperty\">Gibberish.KarplusStrong.amp : property</h3>\n\n<p>Number. A linear value specifying relative amplitude, ostensibly from 0..1 but can be higher, or lower when used for modulation.</p>","channels":"<h3 id=\"gibberishkarplusstrongchannelsproperty\">Gibberish.KarplusStrong.channels : property</h3>\n\n<p>Number. Default 2. If two channels, the signal may be panned.</p>","pan":"<h3 id=\"gibberishkarplusstrongpanproperty\">Gibberish.KarplusStrong.pan : property</h3>\n\n<p>Number. Default 0. The position in the stereo spectrum for the sound, from -1..1.</p>"},"type":" Physical Model"},"Gibberish.Sampler":{"text":"<h1 id=\"gibberishsampleroscillator\">Gibberish.Sampler </h1>\n\n<p>Sample recording and playback.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>Gibberish.init(); <br />\na = new Gibberish.Sampler({ file:'resources/snare.wav' }).connect(); <br />\na.note(2); <br />\na.note(1); <br />\na.note(-.5); <br />\nb = new Gibberish.Sampler().connect(); <br />\nb.record(a, 88200); // record two seconds of a playing <br />\na.note(8); <br />\n// wait a bit <br />\nb.note(1);</code>\n`</p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<h3 id=\"syntax1\">syntax 1</h3>\n\n<p><strong>param</strong> <em>filepath</em>: String. A path to the audiofile to be opened by the sampler.  </p>\n\n<h3 id=\"syntax2\">syntax 2</h3>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the sampler on initialization.</p>\n\n<hr />","methods":{"_onload":"<h3 id=\"gibberishsampler_onloadmethod\">Gibberish.Sampler._onload : method</h3>\n\n<p>This is an event handler that is called when a sampler has finished loading an audio file  </p>\n\n<p>param <strong>buffer</strong> Object. The decoded sampler buffers from the audio file</p>","download":"<h3 id=\"gibberishsamplerdownloadmethod\">Gibberish.Sampler.download : method</h3>\n\n<p>Download the sampler buffer as a .wav file. In conjunction with the record method, this enables the Sampler\nto record and downlaod Gibberish sessions.</p>","note":"<h3 id=\"gibberishsamplernotemethod\">Gibberish.Sampler.note : method</h3>\n\n<p>Trigger playback of the samplers buffer</p>\n\n<p>param <strong>pitch</strong> Number. The speed the sample is played back at. <br />\nparam <strong>amp</strong> Number. Optional. The volume to use.</p>","record":"<h3 id=\"gibberishsamplerrecordmethod\">Gibberish.Sampler.record : method</h3>\n\n<p>Record the output of a Gibberish ugen for a given amount of time</p>\n\n<p>param <strong>ugen</strong> Object. The Gibberish ugen to be recorded.\nparam <strong>recordLength</strong> Number (in samples). How long to record for.</p>","getBuffer":"<h3 id=\"gibberishsamplergetbuffermethod\">Gibberish.Sampler.getBuffer : method</h3>\n\n<p>Returns a pointer to the Sampler's internal buffer.  </p>","callback":"<h3 id=\"gibberishsamplercallbackmethod\">Gibberish.Sampler.callback : method</h3>\n\n<p>Return a single sample. It's a pretty lengthy method signature, they are all properties that have already been listed:  </p>\n\n<p>_pitch, amp, isRecording, isPlaying, input, length, start, end, loops, pan</p>"},"properties":{"pitch":"<h3 id=\"gibberishsamplerpitchproperty\">Gibberish.Sampler.pitch : property</h3>\n\n<p>Number. The speed that the sample is played back at. A pitch of 1 means the sample plays forward at speed it was recorded at, a pitch of -4 means the sample plays backwards at 4 times the speed it was recorded at.</p>","amp":"<h3 id=\"gibberishsamplerampproperty\">Gibberish.Sampler.amp : property</h3>\n\n<p>Number. A linear value specifying relative amplitude, ostensibly from 0..1 but can be higher, or lower when used for modulation.</p>","isRecording":"<h3 id=\"gibberishsamplerisrecordingproperty\">Gibberish.Sampler.isRecording : property</h3>\n\n<p>Boolean. Tells the sample to record into it's buffer. This is handled automatically by the object; there is no need to manually set this property.</p>","isPlaying":"<h3 id=\"gibberishsamplerisplayingproperty\">Gibberish.Sampler.isPlaying : property</h3>\n\n<p>Number. 0..1. Tells the sample to record into it's buffer. This is handled automatically by the object; there is no need to manually set this property.</p>","input":"<h3 id=\"gibberishsamplerinputproperty\">Gibberish.Sampler.input : property</h3>\n\n<p>Object. The object the sampler is tapping into and recording.</p>","length":"<h3 id=\"gibberishsamplerlengthproperty\">Gibberish.Sampler.length : property</h3>\n\n<p>Number. The length of the Sampler's buffer.</p>","start":"<h3 id=\"gibberishsamplerstartproperty\">Gibberish.Sampler.start : property</h3>\n\n<p>Number. When the Sampler's note method is called, sample playback begins at this sample.</p>","end":"<h3 id=\"gibberishsamplerendproperty\">Gibberish.Sampler.end : property</h3>\n\n<p>Number. When the Sampler's note method is called, sample playback ends at this sample.</p>","loops":"<h3 id=\"gibberishsamplerloopsproperty\">Gibberish.Sampler.loops : property</h3>\n\n<p>Boolean. When true, sample playback loops continuously between the start and end property values.</p>","pan":"<h3 id=\"gibberishsamplerpanproperty\">Gibberish.Sampler.pan : property</h3>\n\n<p>Number. -1..1. Position of the Sampler in the stereo spectrum.</p>"},"type":" Oscillator"},"Gibberish":{"text":"<h1 id=\"gibberishmiscellaneous\">Gibberish </h1>\n\n<p>Gibberish is the main object used to manage the audio graph and perform codegen functions. All constructors are also inside of the Gibberish object. Gibberish can automatically generate an appropriate web audio callback for you; if you want to use this you must execute the Gibberish.init() command before creating any Gibberish ugens.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>// make a sine wave <br />\nGibberish.init(); <br />\na = new Gibberish.Sine().connect();</code></p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>bufferSize</em>: Integer. Default 1024. The size of the buffer to be calculated. Since JavaScript is single-threaded, setting exceedingly large values for this will yield to stuttering in graphics and user interface performance.</p>\n\n<hr />","methods":{"createCallback":"<h3 id=\"gibberishcreatecallbackmethod\">Gibberish.createCallback : method</h3>\n\n<p>Perform codegen on all dirty ugens and re-create the audio callback. This method is called automatically in the default Gibberish sample loop whenever Gibberish.isDirty is true.</p>","audioProcess":"<h3 id=\"gibberishaudioprocessmethod\">Gibberish.audioProcess : method</h3>\n\n<p>The default audio callback used in Webkit browsers. This callback starts running as soon as Gibberish.init() is called.  </p>\n\n<p>param <strong>Audio Event</strong> : Object. The HTML5 audio event object.</p>","audioProcessFirefox":"<h3 id=\"gibberishaudioprocessfirefoxmethod\">Gibberish.audioProcessFirefox : method</h3>\n\n<p>The default audio callback used in Firefox. This callback starts running as soon as Gibberish.init() is called.  </p>\n\n<p>param <strong>Sound Data</strong> : Object. The buffer of audio data to be filled</p>","clear":"<h3 id=\"gibberishclearmethod\">Gibberish.clear : method</h3>\n\n<p>Remove all objects from Gibberish graph and perform codegen... kills all running sound and CPU usage.</p>","dirty":"<h3 id=\"gibberishdirtymethod\">Gibberish.dirty : method</h3>\n\n<p>Tell Gibberish a ugen needs to be codegen'd and mark the entire callback as needing regeneration  </p>\n\n<p>param <strong>Ugen</strong> : Object. The ugen that is 'dirtied'... that has a property value changed.</p>","generateSymbol":"<h3 id=\"gibberishgeneratesymbolmethod\">Gibberish.generateSymbol : method</h3>\n\n<p>Generate a unique symbol for a given ugen using its name and a unique id number.  </p>\n\n<p>param <strong>name</strong> : String. The name of the ugen; for example, reverb, delay etc.</p>","AudioDataDestination":"<h3 id=\"gibberishaudiodatadestinationmethod\">Gibberish.AudioDataDestination : method</h3>\n\n<p>Create a callback and start it running. Note that in iOS audio callbacks can only be created in response to user events. Thus, in iOS this method assigns an event handler to the HTML body that creates the callback as soon as the body is touched; at that point the event handler is removed. </p>","makePanner":"<h3 id=\"gibberishmakepannermethod\">Gibberish.makePanner : method</h3>\n\n<p>Create and return an object that can be used to pan a stereo source.</p>","polyInit":"<h3 id=\"gibberishpolyinitmethod\">Gibberish.polyInit : method</h3>\n\n<p>For ugens with polyphony, add metaprogramming that passes on property changes to the 'children' of the polyphonic object. Polyphonic ugens in Gibberish are just single instances that are routed into a shared bus, along with a few special methods for voice allocation etc.  </p>\n\n<p>param <strong>Ugen</strong> : Object. The polyphonic ugen</p>","interpolate":"<h3 id=\"gibberishinterpolatemethod\">Gibberish.interpolate : method</h3>\n\n<p>Similiar to makePanner, this method returns a function that can be used to linearly interpolate between to values. The resulting function takes an array and a floating point position index and returns a value.</p>","ugen":"<h3 id=\"gibberishugenmethod\">Gibberish.ugen : method</h3>\n\n<p>Creates a prototype object that is used by all ugens.</p>"},"properties":{"audioFiles":"<h3 id=\"gibberishaudiofilesproperty\">Gibberish.audioFiles : property</h3>\n\n<p>Array. Anytime an audiofile is loaded (normally using the Sampler ugen) the resulting sample buffer is stored in this array so that it can be immediately recalled.</p>","callback":"<h3 id=\"gibberishcallbackproperty\">Gibberish.callback : property</h3>\n\n<p>String. Whenever Gibberish performs code generation the resulting callback is stored here.</p>","out":"<h3 id=\"gibberishoutproperty\">Gibberish.out : property</h3>\n\n<p>Object. The is the 'master' bus that everything eventually gets routed to if you're using the auto-generated calback. This bus is initialized in the call to Gibberish.init.</p>","dirtied":"<h3 id=\"gibberishdirtiedproperty\">Gibberish.dirtied : property</h3>\n\n<p>Array. A list of objects that need to be codegen'd</p>","isDirty":"<h3 id=\"gibberishisdirtyproperty\">Gibberish.isDirty : property</h3>\n\n<p>Booelan. Whether or codegen should be performed.</p>","codeblock":"<h3 id=\"gibberishcodeblockproperty\">Gibberish.codeblock : property</h3>\n\n<p>Array. During codegen, each ugen's codeblock is inserted into this array. Once all the ugens have codegen'd, the array is concatenated to form the callback.</p>","upvalues":"<h3 id=\"gibberishupvaluesproperty\">Gibberish.upvalues : property</h3>\n\n<p>Array. Each ugen's callback function is stored in this array; the contents of the array become upvalues to the master callback function when it is codegen'd.</p>","debug":"<h3 id=\"gibberishdebugproperty\">Gibberish.debug : property</h3>\n\n<p>Boolean. Default false. When true, the callbackString is printed to the console whenever a codegen is performed</p>","memo":"<h3 id=\"gibberishmemoproperty\">Gibberish.memo : property</h3>\n\n<p>Object. Used in the codegen process to make sure codegen for each ugen is only performed once.</p>"},"type":" Miscellaneous"},"Ugen":{"text":"<h1 id=\"ugenmiscellaneous\">Ugen </h1>\n\n<p>The prototype object that all ugens inherit from</p>","methods":{"processProperties":"<h3 id=\"ugenprocesspropertiesmethod\">Ugen.processProperties : method</h3>\n\n<p>Used to assign and process arguments passed to the constructor functions of ugens.  </p>\n\n<p>param <strong>argumentList</strong> : Array. A list of arguments (may be a single dictionary) passed to a ugen constructor.</p>","codegen":"<h3 id=\"ugencodegenmethod\">Ugen.codegen : method</h3>\n\n<p>Generates output code (as a string) used inside audio callback</p>","getCodeblock":"<h3 id=\"ugengetcodeblockmethod\">Ugen.getCodeblock : method</h3>\n\n<p>Retrieves codeblock for ugen previously created with codegen method.</p>","defineUgenProperty":"<h3 id=\"ugendefineugenpropertymethod\">Ugen.defineUgenProperty : method</h3>\n\n<p>Creates getters and setters for ugen properties that automatically dirty the ugen whenever the property value is changed.  </p>\n\n<p>param <strong>key</strong> : String. The name of a property to add getter / setters for. <br />\nparam <strong>value</strong> : Any. The initival value to set the property to</p>","init":"<h3 id=\"ugeninitmethod\">Ugen.init : method</h3>\n\n<p>Initialize ugen by calling defineUgenProperty for every key in the ugen's properties dictionary, generating a unique id for the ugen and various other small tasks.</p>","mod":"<h3 id=\"ugenmodmethod\">Ugen.mod : method</h3>\n\n<p>Modulate a property of a ugen on a per-sample basis.  </p>\n\n<p>param <strong>key</strong> : String. The name of the property to modulate <br />\nparam <strong>value</strong> : Any. The object or number value to modulate the property with <br />\nparam <strong>op</strong> : String. Default \"+\". The operation to perform. Can be +,-,*,/,= or ++. ++ adds and returns the absolute value.</p>","removeMod":"<h3 id=\"ugenremovemodmethod\">Ugen.removeMod : method</h3>\n\n<p>Remove a modulation from a ugen.  </p>\n\n<p>param <strong>key</strong> : String. The name of the property to remove the modulation from <br />\nparam <strong>arg</strong> : Number or Object. Optional. This determines which modulation to remove if more than one are assigned to the property. If this argument is undefined, all modulations are removed. If the argument is a number, the number represents a modulation in the order that they were applied (an array index). If the argument is an object, it removes a modulation that\nis using a matching object as the modulator.</p>","polyMod":"<h3 id=\"ugenpolymodmethod\">Ugen.polyMod : method</h3>\n\n<p>Applies a modulation to all children of a polyphonic ugen  </p>\n\n<p>param <strong>key</strong> : String. The name of the property to modulate <br />\nparam <strong>value</strong> : Any. The object or number value to modulate the property with <br />\nparam <strong>op</strong> : String. Default \"+\". The operation to perform. Can be +,-,*,/,= or ++. ++ adds and returns the absolute value.</p>","removePolyMod":"<h3 id=\"ugenremovepolymodmethod\">Ugen.removePolyMod : method</h3>\n\n<p>Removes a modulation from all children of a polyphonic ugen. The arguments  </p>\n\n<p>param <strong>arg</strong> : Number or Object. Optional. This determines which modulation to remove if more than one are assigned to the property. If this argument is undefined, all modulations are removed. If the argument is a number, the number represents a modulation in the order that they were applied (an array index). If the argument is an object, it removes a modulation that\nis using a matching object as the modulator.</p>","connect":"<h3 id=\"ugenconnectmethod\">Ugen.connect : method</h3>\n\n<p>Connect the output of a ugen to a bus.  </p>\n\n<p>param <strong>bus</strong> : Bus ugen. Optional. The bus to connect the ugen to. If no argument is passed the ugen is connect to Gibberish.out. Gibberish.out is automatically created when Gibberish.init() is called and can be thought of as the master stereo output for Gibberish.</p>","send":"<h3 id=\"ugensendmethod\">Ugen.send : method</h3>\n\n<p>Send an arbitrary amount of output to a bus  </p>\n\n<p>param <strong>bus</strong> : Bus ugen. The bus to send the ugen to. <br />\nparam <strong>amount</strong> : Float. The amount of signal to send to the bus. </p>","disconnect":"<h3 id=\"ugendisconnectmethod\">Ugen.disconnect : method</h3>\n\n<p>Disconnect a ugen from a bus (or all busses). This stops all audio and signal processing for the ugen.  </p>\n\n<p>param <strong>bus</strong> : Bus ugen. Optional. The bus to send the ugen to. If this argument is undefined the ugen will be disconnected from all busses.</p>"},"properties":{},"type":" Miscellaneous"},"Gibberish.Bus":{"text":"<h1 id=\"gibberishbusmiscellaneous\">Gibberish.Bus </h1>\n\n<p>Create a mono routing bus. A bus callback routes all it's inputs and scales them by the amplitude of the bus.  </p>\n\n<p>For a stereo routing bus, see <a href=\"javascript:displayDocs('Gibberish.Bus2')\">Bus2</a></p>\n\n<h2 id=\"exampleusage\">Example Usage##</h2>\n\n<p><code>a = new Gibberish.Bus(); <br />\nb = new Gibberish.Sine(440).connect(a); <br />\nc = new Gibberish.Sine(880).connect(a); <br />\na.amp = .1; <br />\na.connect();</code></p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the bus on initialization.</p>","methods":{},"properties":{"amp":"<h3 id=\"gibberishbusampproperty\">Gibberish.Bus.amp : property</h3>\n\n<p>Array. Read-only. Relative volume for the sum of all ugens connected to the bus.</p>"},"type":" Miscellaneous"},"Gibberish.Bus2":{"text":"<h1 id=\"gibberishbus2miscellaneous\">Gibberish.Bus2 </h1>\n\n<p>Create a stereo outing bus. A bus callback routes all it's inputs and scales them by the amplitude of the bus.</p>\n\n<h2 id=\"exampleusage\">Example Usage##</h2>\n\n<p>`a = new Gibberish.Bus2(); <br />\nb = new Gibberish.Sine(440).connect(a); <br />\nc = new Gibberish.Sine(880).connect(a);  </p>\n\n<p>d = new Gibberish.Sequencer({ target:a, key:'pan', values:[-.75,.75], durations:[ 22050 ] }).start();\na.connect();`</p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the bus on initialization.</p>","methods":{},"properties":{},"type":" Miscellaneous"},"Gibberish.Sequencer":{"text":"<h1 id=\"gibberishsequencermiscellaneous\">Gibberish.Sequencer </h1>\n\n<p>A sample-accurate sequencer that can sequence changes to properties, method calls or anonymous function calls.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>Gibberish.init(); <br />\na = new Gibberish.Synth({ attack:44, decay:44100 }).connect(); <br />\nb = new Gibberish.Sequencer({ target:a, key:'note', durations:[11025, 22050], values:[440, 880] }).start()\n</code>  </p>\n\n<h2 id=\"constructor\">Constructor</h2>\n\n<p><strong>param</strong> <em>properties</em>: Object. A dictionary of property values (see below) to set for the sequencer on initialization.</p>\n\n<hr />","methods":{"start":"<h3 id=\"gibberishsequencerstartmethod\">Gibberish.Sequencer.start : method</h3>\n\n<p>Start the sequencer running.</p>\n\n<p>param <strong>shouldKeepOffset</strong> boolean, default false. If true, the phase of the sequencer will not be reset when calling the start method.</p>","stop":"<h3 id=\"gibberishsequencerstopmethod\">Gibberish.Sequencer.stop : method</h3>\n\n<p>Stop the sequencer.</p>","repeat":"<h3 id=\"gibberishsequencerrepeatmethod\">Gibberish.Sequencer.repeat : method</h3>\n\n<p>Play the sequencer a certain number of times and then stop it.</p>\n\n<p>param <strong>timesToRepeat</strong> number. The number of times to repeat the sequence.</p>","disconnect":"<h3 id=\"gibberishsequencerdisconnectmethod\">Gibberish.Sequencer.disconnect : method</h3>\n\n<p>Each sequencer object has a tick method that is called once per sample. Use the disconnect method to stop the tick method from being called.</p>","connect":"<h3 id=\"gibberishsequencerconnectmethod\">Gibberish.Sequencer.connect : method</h3>\n\n<p>Each sequencer object has a tick method that is called once per sample. Use the connect method to start calling the tick method. Note that the connect\nmethod is called automatically when the sequencer is first created; you should only need to call it again if you call the disconnect method at some point.</p>"},"properties":{"target":"<h3 id=\"gibberishsequencertargetproperty\">Gibberish.Sequencer.target : property</h3>\n\n<p>Object. An object for the sequencer to control. May be null if you are sequencing anonymous functions.</p>","key":"<h3 id=\"gibberishsequencerkeyproperty\">Gibberish.Sequencer.key : property</h3>\n\n<p>String. The name of the method or property you would like to sequnce on the Sequencer's target object.</p>","durations":"<h3 id=\"gibberishsequencerdurationsproperty\">Gibberish.Sequencer.durations : property</h3>\n\n<p>Array. The number of samples between each advancement of the Sequencer. Once the Sequencer arrives at the end of this list, it loops back to the beginning</p>","keysAndValues":"<h3 id=\"gibberishsequencerkeysandvaluesproperty\">Gibberish.Sequencer.keysAndValues : property</h3>\n\n<p>Object. A dictionary holding a set of values to be sequenced. The keys of the dictionary determine which methods and properties to sequence on the Sequencer's target object and\neach key has an array of values representing the sequence for that key.</p>\n\n<p><code>Gibberish.init(); <br />\na = new Gibberish.Synth({ attack:44, decay:44100 }).connect(); <br />\nb = new Gibberish.Sequencer({ target:a, durations:[11025, 22050], keysAndValues:{ 'note':[440,880], 'amp':[.2,.4] } }).start()\n</code></p>"},"type":" Miscellaneous"},"Gibberish.Time":{"text":"<h1 id=\"gibberishtimemiscellaneous\">Gibberish.Time </h1>\n\n<p>This object is used to simplify timing in Gibberish. It contains an export function to place its methods in another object (like window)\nso that you can code more tersely. The methods of the Time object translate ms, seconds and beats into samples. The default bpm is 120.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>Gibberish.Time.export(); // now all methods are also part of the window object\na = new Gibberish.Sine(440).connect(); <br />\nb = new Gibberish.Sequencer({ target:a, key:'frequency', durations:[ seconds(1), ms(500), beats( .5 ) ], values:[220,440,880] }).start() <br />\n</code></p>","methods":{"export":"<h3 id=\"gibberishtimeexportmethod\">Gibberish.Time.export : method</h3>\n\n<p>Use this to export the methods of Gibberish.Time so that you can tersely refer to them.</p>\n\n<p>param <strong>target</strong> object, default window. The object to export the Gibberish.Time methods into.</p>","ms":"<h3 id=\"gibberishtimemsmethod\">Gibberish.Time.ms : method</h3>\n\n<p>Convert the parameter from milliseconds to samples.</p>\n\n<p>param <strong>ms</strong> number. The number of milliseconds to convert.</p>","seconds":"<h3 id=\"gibberishtimesecondsmethod\">Gibberish.Time.seconds : method</h3>\n\n<p>Convert the parameter from seconds to samples.</p>\n\n<p>param <strong>seconds</strong> number. The number of seconds to convert.</p>","beats":"<h3 id=\"gibberishtimebeatsmethod\">Gibberish.Time.beats : method</h3>\n\n<p>Return a function that converts the parameter from beats to samples. This method uses the bpm property of the Gibberish.Time object to determine the duration of a sample.\nYou can use the function returned by this method in a Sequencer; if Gibberish.Time.bpm is changed before the function is executed the function will use the updated value.</p>\n\n<p>param <strong>seconds</strong> number. The number of seconds to convert.</p>"},"properties":{"bpm":"<h3 id=\"gibberishtimebpmproperty\">Gibberish.Time.bpm : property</h3>\n\n<p>Number. Default 120. The beats per minute setting used whenever a call to the beats method is made.</p>"},"type":" Miscellaneous"},"Gibberish.Binops":{"text":"<h1 id=\"gibberishbinopsmiscellaneous\">Gibberish.Binops </h1>\n\n<p>These objects create binary operations - mathematical operations taking two arguments - and create signal processing functions using them. They are primarily used for\nmodulation purposes. You can export the constructors for easier use similar to the <a href=\"javascript:displayDocs('Gibberish.Time')\">Time</a> constructors.</p>\n\n<p>Add, Sub, Mul and Div can actually take as many arguments as you wish. For example, Add(1,2,3,4) will return an object that outputs 10. You can stack multiple oscillators this way as well.</p>\n\n<h2 id=\"exampleusage\">Example Usage</h2>\n\n<p><code>// This example creates a tremolo effect via amplitude modulation <br />\nGibberish.Binops.export(); // now all constructors are also part of the window object <br />\nmod = new Gibberish.Sine(4, .25); <br />\nsin = new Gibberish.Sine( 440, add( .5, mod ) ).connect(); <br />\n</code></p>","methods":{"export":"<h3 id=\"gibberishbinopsexportmethod\">Gibberish.Binops.export : method</h3>\n\n<p>Use this to export the constructor methods of Gibberish.Binops so that you can tersely refer to them.</p>\n\n<p>param <strong>target</strong> object, default window. The object to export the Gibberish.Binops constructors into.</p>","Add":"<h3 id=\"gibberishbinopsaddmethod\">Gibberish.Binops.Add : method</h3>\n\n<p>Create an object that sums all arguments at audio rate. The arguments may be unit generators, numbers, or any mix of the two.</p>","Sub":"<h3 id=\"gibberishbinopssubmethod\">Gibberish.Binops.Sub : method</h3>\n\n<p>Create an object that starts with the first argument and subtracts all subsequent arguments at audio rate. The arguments may be unit generators, numbers, or any mix of the two.</p>","Mul":"<h3 id=\"gibberishbinopsmulmethod\">Gibberish.Binops.Mul : method</h3>\n\n<p>Create an object that calculates the product of all arguments at audio rate. The arguments may be unit generators, numbers, or any mix of the two.</p>","Div":"<h3 id=\"gibberishbinopsdivmethod\">Gibberish.Binops.Div : method</h3>\n\n<p>Create an object that takes the first argument and divides it by all subsequent arguments at audio rate. The arguments may be unit generators, numbers, or any mix of the two.</p>","Mod":"<h3 id=\"gibberishbinopsmodmethod\">Gibberish.Binops.Mod : method</h3>\n\n<p>Create an object that takes the divides the first argument by the second and returns the remainder at audio rate. The arguments may be unit generators, numbers, or any mix of the two.</p>","Abs":"<h3 id=\"gibberishbinopsabsmethod\">Gibberish.Binops.Abs : method</h3>\n\n<p>Create an object that returns the absolute value of the (single) argument. The argument may be a unit generator or number.</p>","Sqrt":"<h3 id=\"gibberishbinopssqrtmethod\">Gibberish.Binops.Sqrt : method</h3>\n\n<p>Create an object that returns the square root of the (single) argument. The argument may be a unit generator or number.</p>","Pow":"<h3 id=\"gibberishbinopspowmethod\">Gibberish.Binops.Pow : method</h3>\n\n<p>Create an object that returns the first argument raised to the power of the second argument. The arguments may be a unit generators or numbers.</p>"},"properties":{},"type":" Miscellaneous"}}